/*---------------------------------------------------------------------------
  |   ModulName:   P_Menu.c                   Version:  0.8.0               |
  ---------------------------------------------------------------------------
  |     Projekt:   Menu H8 <-> Hyperterminal  Technikerschule USTER         |
  |                                                                         |
  |     Autoren:   Benno Huebscher (HU)       reselec ag                    |
  |                Hanno Baumgartner (HB)     evis ag                       |
  |                                                                         |
  |       Datum:   03.11.2001                                               |
  |                                                                         |
  ----- Beschreibung --------------------------------------------------------
  |                                                                         |
  |  1. Interruptgesteuerte RS - Schnittstelle                              |
  |  2. Menu auf Hyperterminal COM1 zur Steuerung und Abfrage von div.      |
  |     Parametern.                                                         |
  |                                                                         |
  |                                                                         |
  |  !!! do never use: strcpy([string], ""); (blank string doesn't work)!!! |
  |                                                                         |
  ----- History -------------------------------------------------------------
  |   03.11.2001   version 0.5 created                       B. Huebscher   |
  |   03.12.2001   Interrupt controlled RS1 Interface added  H. Baumgartner |
  |   03.12.2001   version 0.6 created                       B. Huebscher   |
  |   04.12.2001   Interrupt controlled Keyboard added		 H. Baumgartner |
  |   07.12.2001   div. comments                             B. Huebscher   |
  |   07.12.2001   version 0.6 created                       B. Huebscher   |
  ---------------------------------------------------------------------------*/

//--------------------------------------------------------------------------------------------------
#include "CB-TSU2.h"
#include "ioh83067.h"
#include <stdio.h>
#include <string.h>

/* Define Datatypes */
#define BYTE unsigned char
#define WORD unsigned int
#define LONG unsigned long

#define TRUE  1
#define FALSE 0
#define EXIT_SUCCESS 1
#define EXIT_FAILURE 0

/* Input Characters  */
#define CR 0x0D     // Carrige Return
#define LF 0x0A     // Line Feed

/* Output Characters  */
#define MAX_RS_OUTPUT_STRING_LENGTH     4000     // max. Output String for Menu Terminal
#define MAX_RS_INPUT_STRING_LENGTH      21       // max. Input String for Menu Terminal
#define MAX_MENU_TITLE_STRING_LENGTH    20       // max. Output String for Menu Title 
#define MAX_LCD_DISP_STRING_LENGTH      21       // max. Output String for LCD Display

/* Peripherial Definitions */
#define Switch   (*(volatile unsigned char*)0x400030)
#define LEDgrun  (*(volatile unsigned char*)0x400040)
#define LEDrot   (*(volatile unsigned char*)0x400050)
//--------------------------------------------------------------------------------------------------


/* Globals */
//--------------------------------------------------------------------------------------------------
volatile long  time_val;                                // Time values for internal clock
BYTE  hundertstel;                                      //
BYTE  sekunden;                                         //
BYTE  minuten;                                          //
BYTE  stunden;                                          //

BYTE  dummy;                                            // Dummy Byte to safe registers
BYTE  n = 0;                                            // Aktuelle Position im RS_Daten Array
BYTE  Valid_String;                                     // Info ob ein gltiger String vorhanden ist (1=ja)
BYTE  pos;                                              // Hilfsvar. zur Orientierung der Pos. im RS_Daten Array
BYTE  dummy;                                            // Hilfsvariabel zum lesen der Register
BYTE  KeyboardVal;                                      // effektiv eingelesener Keyboard-Wert 
BYTE  KeyboardIntVal;                                   // Tastatur Variable fr Interrupt Routine
char  RS_IN_Buffer    [ MAX_RS_INPUT_STRING_LENGTH ];   // Sicherungsbuffer fr RS Daten             
char  RS_Daten        [ MAX_RS_INPUT_STRING_LENGTH ];   // Empfangsbuffer fr RS Daten               
char  buffer          [ MAX_LCD_DISP_STRING_LENGTH ];   // Grsse Entspricht der Anz. Zeichen pro Zeile

char  LCDTimeBuffer   [ MAX_LCD_DISP_STRING_LENGTH ];   // Output Buffer Time Value on LCD 
char  LCDIntBuffer    [ MAX_LCD_DISP_STRING_LENGTH ];   // Output Buffer Interrupt message on LCD
char  LCDCmdBuffer    [ MAX_LCD_DISP_STRING_LENGTH ];   // Output Buffer Command on LCD
char  RS_IN_Buffer    [ MAX_RS_INPUT_STRING_LENGTH ];   // Input String to Menu
char  RS_OUT_Buffer   [ MAX_RS_OUTPUT_STRING_LENGTH ];  // Output String to Menu

 
 
/* States Definitions */
//--------------------------------------------------------------------------------------------------
enum  enum_states                   // Definition States of StateMachine
{
	ST_UNDEFINED,

	ST_INIT,

	ST_Menu_0,

	ST_Menu_1,
	ST_Menu_2,
	ST_Menu_3,
	ST_Menu_4,
	ST_Menu_5,

	ST_Menu_1_1,
	ST_Menu_1_1_1,
	ST_Menu_1_1_2,
	ST_Menu_1_1_3,
	ST_Menu_1_1_4,
	ST_Menu_1_1_5,

	ST_Menu_1_2,
	ST_Menu_1_3,
	ST_Menu_1_4,
	ST_Menu_1_5,

	ST_Menu_2_1,
	ST_Menu_2_2,
	ST_Menu_2_3,
	ST_Menu_2_4,
	ST_Menu_2_5,

	ST_Menu_3_1,
	ST_Menu_3_2,
	ST_Menu_3_3,
	ST_Menu_3_4,
	ST_Menu_3_5,

	ST_Menu_4_1,
	ST_Menu_4_2,
	ST_Menu_4_3,
	ST_Menu_4_4,
	ST_Menu_4_5,

	ST_Menu_5_1,
	ST_Menu_5_2,
	ST_Menu_5_3,
	ST_Menu_5_4,
	ST_Menu_5_5,

	ST_LAST
};
//--------------------------------------------------------------------------------------------------


/* Prototypes */
//--------------------------------------------------------------------------------------------------
int  Menu_print_all ( char s[MAX_RS_OUTPUT_STRING_LENGTH] );
int  Menu_print_by_Menu ( int par_LevelNo, int par_MenuNo, char s[MAX_RS_OUTPUT_STRING_LENGTH]);
char *Menu_getMenuTitle(int par_LevelNo, int par_MenuNo);

int Menu_handler( void );
static int Menu_do_init    ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_0  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_1  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_1_1( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_1_2( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_2  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_3  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_4  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
static int Menu_do_Menu_5  ( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] );
 
void Save_RS_Data    ( void );

void calculate_clock ( void );

void LED_Rot_ausg    ( char LED_contr );
void LED_Gruen_ausg  ( char LED_contr );
void Switch_anz      ( char Switch_contr );
void Keyboard_disp   ( void );
//--------------------------------------------------------------------------------------------------




// Datafield for Menutext
//--------------------------------------------------------------------------------------------------
typedef struct SMenu {
	int  LevelNo;
	int  MenuNo;
	char MenuText [ 14 ];   // max 13. char per Menuname
	int  PointNo;
	char PointText[ 16 ];   // max 15. char per Pointname
} TMenu;

#define TOTAL_MENU_Points 60

TMenu MenuDefs[TOTAL_MENU_Points]= {

// only the first Level- and Menu-Names are important !!!
// __________________________________________________
// Level|Menu|  MenuTitle  |Point|    PointName     |
//      |    |             |     |                  |
	{ 1,  0, "   H8-Menu   ",  1, "LED's          "},
	{ 1,  0, "             ",  2, "Switches       "},
	{ 1,  0, "             ",  3, "LCD-Display    "},
	{ 1,  0, "             ",  4, "Keyboard       "},
	{ 1,  0, "             ",  5, "Menu Infos     "},
	{ 2,  1, "   LED's     ",  1, "LED's green    "},
	{ 2,  1, "             ",  2, "LED's red      "},
	{ 3, 11, " LED's green ",  1, "LED green 0    "},
	{ 3, 11, "             ",  2, "LED green 1    "},
	{ 3, 11, "             ",  3, "LED green 2    "},  // 10
	{ 3, 11, "             ",  4, "LED green 3    "},
	{ 3, 11, "             ",  5, "LED green 4    "},
	{ 3, 11, "             ",  6, "LED green 5    "},
	{ 3, 11, "             ",  7, "LED green 6    "},
	{ 3, 11, "             ",  8, "LED green 7    "},
	{ 3, 11, "             ",  9, "all LED's off  "},
	{ 3, 12, " LED's red   ",  1, "LED red 0      "},
	{ 3, 12, "             ",  2, "LED red 1      "},
	{ 3, 12, "             ",  3, "LED red 2      "},
	{ 3, 12, "             ",  4, "LED red 3      "},  // 20
	{ 3, 12, "             ",  5, "LED red 4      "},
	{ 3, 12, "             ",  6, "LED red 5      "},
	{ 3, 12, "             ",  7, "LED red 6      "},
	{ 3, 12, "             ",  8, "LED red 7      "},
	{ 3, 12, "             ",  9, "all LED's off  "},  
	{ 2,  2, "  Switches   ",  1, "Display SW Dec "},
	{ 2,  2, "             ",  2, "Display SW Hex "},
	{ 2,  2, "             ",  3, "Display SW Bin "},
	{ 2,  2, "             ",  4, "Display HexSw D"},
	{ 2,  2, "             ",  5, "Display HexSw H"},  // 30
	{ 2,  2, "             ",  6, "Display HexSw B"},
	{ 2,  3, " LCD-Display ",  1, "Backlight  on  "},
	{ 2,  3, "             ",  2, "Backlight  off "},
	{ 2,  3, "             ",  3, "-              "},
	{ 2,  3, "             ",  4, "-              "},
	{ 2,  3, "             ",  5, "-              "},
	{ 2,  3, "             ",  6, "-              "},
	{ 2,  4, "  Keyboard   ",  1, "Display Keys..."},
	{ 2,  4, "             ",  2, "-              "},
	{ 2,  4, "             ",  3, "-              "},  // 40
	{ 2,  4, "             ",  4, "-              "},
	{ 2,  4, "             ",  5, "-              "},
	{ 2,  5, "System-Infos ",  1, "print Menu     "},
	{ 2,  5, "             ",  2, "about...       "},
	{ 2,  5, "             ",  3, "clock          "},
	{ 2,  5, "             ",  4, "-              "},
	{ 2,  5, "             ",  5, "-              "},
	{ 0,  0, "no Menu      ",  1, "DummyMenuPoint "},
	{ 0,  0, "no Menu      ",  1, "DummyMenuPoint "},
	{ 0,  0, "no Menu      ",  1, "DummyMenuPoint "},  // 50
	{ 0,  0, "no Menu      ",  1, "DummyMenuPoint "},  // at least one Dummy Element !!!
//      |    |             |     |                  |
// Level|Menu|  MenuTitle  |Point|    PointName     |
// --------------------------------------------------
};
//--------------------------------------------------------------------------------------------------




/* INTERRUPT ROUTINES */
/**********************/

//--------------------------------------------------------------------------------------------------
// Internal Clock Counter0 Interrupt
//--------------------------------------------------------------------------------------------------
#pragma interrupt
void ISR_tic(void)              // clock Timer
{
	dummy     = FRT_TISRA;      // Read Register into dummy element (important)
	FRT_TISRA = 0x88 + 0x10;    // Enable Interrupt
	time_val ++;                // increase Time Value
}   // End Interrupt ISR_tic


//--------------------------------------------------------------------------------------------------
// SCI Error Handling Interrupt Routine
//--------------------------------------------------------------------------------------------------
#pragma interrupt
void ISR_ERI1( void )
{
	BYTE  errors;
	
	errors = SCI_SSR1;    // Registerzustand in errors sichern
	
	errors &= 0x38;       // Ausmaskierung der Errormeldungen
	switch(errors)
	{
		case 0x20 :  CBTSU_DispString(2,"***Overrun Error*** ");  break;
		case 0x10 :  CBTSU_DispString(2,"***Framing Error*** ");  break;
		case 0x08 :  CBTSU_DispString(2,"***Parity Error***  ");  break;
		case 0x30 :  CBTSU_DispString(2,"*Over + Fram Error* ");  break;
		case 0x28 :  CBTSU_DispString(2,"*Over + Pari Error* ");  break;
		case 0x18 :  CBTSU_DispString(2,"*Fram + Pari Error* ");  break;
		case 0x38 :  CBTSU_DispString(2,"*Fr + Ov + Pa Error*");  break; 
	}
	dummy = SCI_SSR1; SCI_SSR1 &= 0x84;     // Errorflags nach Anzeige des Errors lschen
}   // End Interrupt ISR_ERI1                                                  


//--------------------------------------------------------------------------------------------------
// RS Daten interrupt gesteuert einlesen, empf. Zeichen ausgeben, Test auf String
//--------------------------------------------------------------------------------------------------
#pragma interrupt
void ISR_RS( void ) 
{
	if (Valid_String == 0)         // Nur weiterfahren, wenn noch kein gltiger String vorhanden
	{                              // somit wird das berschreiben eines gltigen Strings verhindert
		RS_Daten[n] = SCI_RDR1;    // Empfgangsbyte in RS Daten Array sichern
		
		while ((SCI_SSR1 & 0x80) != 0x80);     // Empfangenes Zeichen ausgeben
		SCI_TDR1 = (unsigned char)RS_Daten[n]; // an Terminalprogramm (RS232)
		SCI_SSR1 &= 0x7F;
		
		n++;                                   // n erhhen um das nchste Zeichen im Array abzuglegen
		pos = n-1;                             // Zeichenposition ist um 1 kleiner als n
		if (n > (MAX_RS_INPUT_STRING_LENGTH - 1))      // wird n zu gross, wird es zurckgesetzt, somit wird 
		{                                      // ein "Rigbuffer" geschaffen, welcher immer an gleicher Stelle
			n = 0;                             // wieder beginnt => Somit wird das zuletzt eingelesene Zeichen 
			pos=MAX_RS_INPUT_STRING_LENGTH - 1;        // nie verpasst, auch nicht wenn anz Zeichen > Befehl_l
		}
		if (RS_Daten[pos]==CR)                 // Wurde die Entertaste gedrckt, so ist der String vollstndig
		{                                      // und bereit zum auswerten
			RS_Daten[pos]='\0';    // Das Abschlusszeichen '\0'dem Array hinzugefgt
			Valid_String=1;
			n=0; 
		}      
	}
    dummy = SCI_SSR1; SCI_SSR1 &= 0x84;   // Interruptbedingung zurcksetzen
}

//*****************************************************************************
// 4x4 Tastatur einlesen
//*****************************************************************************
#pragma interrupt
void ISR_IRQ0(void) 
{
    KeyboardIntVal = CBTSU_Keyboard ;     // Keyboardwert sichern
	KeyboardIntVal = KeyboardIntVal-0xF0; // Offset subtrahieren
	  switch(KeyboardIntVal)              // Gedrckte Taste dem entsprechenden
	  {                                   // Wert zuweisen
	  case 0:	KeyboardVal='1';
	  			break;
	  case 1:	KeyboardVal='2';
	  			break;
	  case 2:	KeyboardVal='3';
	  			break;
	  case 3:	KeyboardVal='F';
	  			break;
	  case 4:	KeyboardVal='4';
	  			break;
	  case 5:	KeyboardVal='5';
	  			break;
	  case 6:	KeyboardVal='6';
	  			break;
	  case 7:	KeyboardVal='E';
	  			break;
	  case 8:	KeyboardVal='7';
	  			break;
	  case 9:	KeyboardVal='8';
	  			break;
	  case 10:	KeyboardVal='9';
	  			break;			
	  case 11:	KeyboardVal='D';
	  			break;
	  case 12:	KeyboardVal='A';
	  			break;
	  case 13:	KeyboardVal='0';
	  			break;
	  case 14:	KeyboardVal='B';
	  			break;
	  case 15:	KeyboardVal='C';
	  			break;																																																
	  }	 
	dummy = ISR; ISR = ISR & 0xfe;        // Lschen der Interruptbedingung
}
                       

/* MAIN ROUTINES */
/*****************/
//--------------------------------------------------------------------------------------------------
// Initialisierung der Hardware => RS 232 Schnittstelle, Display
//--------------------------------------------------------------------------------------------------
void Init_HW (void)
{  
	// Serial Control Register (S 472)
	// Bit7    Bit6    Bit5    Bit4    Bit3    Bit2     Bit1    Bit0
	// 0       0       0       0       0       0        0       0
	// Transm  Receve  Transm  Receve  Muli    Transm   ------------
	// Int     Int     enable  enable  ProzM   End Int  Clock Source
	// Aktuelle Konfiguration:
	// Transmit Interrupt disabled / Receve Int (RXI, ERI) disabled / Transmit and Receve disabled /
	// Multiproz disabled / Transm End Int disabled / Clock Sorurce = Asynch int Clock generic Input(SCK) 
	SCI_SCR1 = 0x00;
	
	//Mode der Seriellen Schnittstelle (S 468)
	// Bit7    Bit6    Bit5    Bit4    Bit3    Bit2    Bit1    Bit 0
	// 0       0       0       0       0       0       0       0
	// Com-    Char-   Parity  Parity  Stop    Multi   -------------
	// Mode    length  enable  Mode    Bit l   prozM   Clock Select
	// Aktuelle Konfiguration:
	// Asynchron / 8Bit Data / Parity not checked / Even Parity / 1Stop Bit / Muliprozessor disabled / Clock=1
	SCI_SMR1 = 0x00;
	
	//Baudrate 9600 => 59dez
	SCI_BRR1 = 0x3B; 
	
	// Serial Status Register SSR (S 477)
	// Bit7    Bit6    Bit5    Bit4    Bit3    Bit2    Bit1    Bit0
	// R/C     R/C     R/C     R/C     R/C     R       R       0
	// Transm  Receve  Overr   Framing Parity  Transm  Multip  Multip Bit
	// D.empty D.full  Error   Error   Error   End     Bit     transfer
	// Aktuelle Konfiguration:
	// Undverknpfung mit Transmit Daten empty Transm End => Initialwert, lschen
	// von evtl. Error
	dummy = SCI_SSR1; SCI_SSR1 &= 0x84; 
	
	// Serial Control Register (S 472)
	// Bit7    Bit6    Bit5    Bit4    Bit3    Bit2     Bit1    Bit0
	// 0       1       1       1       0       0        0       0
	// Transm  Receve  Transm  Receve  Muli    Transm   ------------
	// Int     Int     enable  enable  ProzM   End Int  Clock Source
	// Aktuelle Konfiguration:
	// Transmit Interrupt disabled / Receve Int (RXI, ERI)enabled / Transmit and Receve enabled /
	// Multiproz disabled / Transm End Int disabled / Clock Sorurce = Asynch int Clock generic Input(SCK) 
	SCI_SCR1 = 0x70;
	
	LCD_Init();             // Initialisierung LCD Display
	LCD_IWrite(0x01);
	PBDR |= 0x20;           // Backlight des LCD Displays einschalten
	CBTSU_DispString(1,"TSU H8-Menu   v.0.8 ");  // Ausgabe des Softwaretitels

	CBTSU_DispString(2," a TauGnix(c) and...");  // Ausgabe der SoftwareHelden 
	CBTSU_DispString(3,"   ... Baumgartner  ");  // Ausgabe des SoftwareHelden 
	CBTSU_DispString(4,"   production  2001 ");  // Ausgabe des SoftwareHelden
	
	// Initialisierung Timer
	time_val = 0.0;           // Zeit auf Null setzten
	// GRA als Compare Register aktivieren
	FRT_TIOR0 = 0x88 + 0x00;
	// Timer Compare Wert laden, da 8 Bit Mode in zwei Schritten
	FRT_GRA0L = (BYTE) (23040 % 0x100);
	FRT_GRA0H = (BYTE) (23040 / 0x100);
	// Automatisches Loeschen von TSTR aktivieren & Phi achtel
	FRT_TCR0 = 0x80 + 0x23;
	// Match Flag zur Sicherheit lschen
	dummy =FRT_TISRA; FRT_TISRA = 0x88 + 0x10;
	// Timer starten
	FRT_TSTR = 0xf8 + 0x01;
	// Fallende Flanke
	ISCR = 0x01;
	
	// Ext_int freigeben (Tastatur) 
	IER = 0x01;
}   // En Init_HW
//--------------------------------------------------------------------------------------------------



//--------------------------------------------------------------------------------------------------
//  MAIN PROGRAMM
//--------------------------------------------------------------------------------------------------
int main (void)
{
	BYTE  z; // Zhler fr die Sicherung des RS_Daten Strings
	         // Achtung Grsse muss mindestends gleich wie diejenige von MAX_RS_INPUT_STRING_LENGTH sein!

	H8interrupt	[SCI_RXI1]  H8proc ISR_RS;        // Interrupt for RS-CCOM 1
	H8interrupt	[SCI_ERI1]  H8proc ISR_ERI1;      // Interrupt for RS-ERROR
	H8interrupt [FRT_IMIA0] H8proc ISR_tic;       // Timer Interrupr for internal clock
	H8interrupt [IRQ_0]     H8proc ISR_IRQ0;      // Interrupt Tastatur

	Init_HW();                                    // Hardware initialisation
	
/* MAIN LOOP */
	do {

		calculate_clock ();

		if (Valid_String==1)      // 1 = Ein neuer Befehlssatz muss ausgewertet werden
		{   
			CBTSU_DispString(2,"                    ");  // Lschen einer eventuell vorh. Com Error Meldung
			CBTSU_DispString(3,"                    ");  // Lschen einer eventuell vorh. message

			Save_RS_Data();                              // Int. eingelesener String wird gesichert

			/* Write incomming String to LCD-Display Line 4*/
			strcpy( LCDCmdBuffer, "$ ");
			strcat( LCDCmdBuffer, RS_IN_Buffer);
			CBTSU_DispString(4, "                    ");
			CBTSU_DispString(4, LCDCmdBuffer);           // Achtung Lnge kleiner 20 chars

			/* Menu Handler */
			RS_OUT_Buffer[0] = '\0';                     // clear Output Buffer
			Menu_handler();                              // call Menu handler
			strcat( RS_OUT_Buffer, "\n $ ");             // add Prompt sign to Output-String
			
			/* Write String to RS */
			CBTSU_RSWriteLn( RS_OUT_Buffer );

            Valid_String=0;            // 0 = kein Befehlssatz zum auswerten mehr vorhanden 
			                           // somit kann ber Interrupt wieder ein neuer Befehl
			                           // eingelesen werden
		}
		
		// write here Main Programm....
		
	}
	while(1);      // End MAIN LOOP
}                  // End MAIN
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
// Funktion zum sichern der Interrupt eingelesenen RS_Daten
//--------------------------------------------------------------------------------------------------
void Save_RS_Data (void)
{
	BYTE  z;                                   // Zhler fr die Sicherung des RS_Daten Strings
	                                           // Achtung Grsse muss mindestends gleich wie
                                               // diejenige von Befehl_l sein!

	for (z=0; z<MAX_RS_INPUT_STRING_LENGTH; z++)                 // Sichern des eingelesenen Strings 
	{                      
		RS_IN_Buffer[z]=RS_Daten[z];
	}
	RS_Daten[pos]='0';                         // berschreiben der Endung des alten Strings,
	                                           // somit ist alter String ungltig
}                                              // END Save_RS_Data 

 
//--------------------------------------------------------------------------------------------------
//   calculate clock
//--------------------------------------------------------------------------------------------------
void calculate_clock ( void )
{
		hundertstel = time_val % 100;
		sekunden    = ((time_val / 100)    % 60);
		minuten     = ((time_val / 6000)   % 60);
		stunden     = ((time_val / 360000) % 60);
		sprintf( LCDTimeBuffer, "  time: %02u:%02u:%02u:%02u", stunden, minuten, sekunden, hundertstel);
		//CBTSU_DispString(2, LCDTimeBuffer);
}

     
//--------------------------------------------------------------------------------------------------
//   Print complete Menu
//--------------------------------------------------------------------------------------------------
int Menu_print_all ( char s[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	int  i;
	char buf[100];

	strcpy(s, "\n -------------------------------------------------------\n");
	strcat(s, " Level  Menu    Menu Name   Point   Point Name  \n");
	strcat(s, " -------------------------------------------------------\n");

	strcpy(buf, "");

	for ( i=0; i<TOTAL_MENU_Points; i++ )
	{
		if (MenuDefs[i].LevelNo != 0)
		{
			sprintf(buf, "  %4d  %4d  %s  %4d   %s\n",
				MenuDefs[i].LevelNo,
				MenuDefs[i].MenuNo,
				MenuDefs[i].MenuText,
				MenuDefs[i].PointNo,
				MenuDefs[i].PointText
			);
			strcat(s, buf);
		};
	}
	strcat(s, " -------------------------------------------------------\n");
	sprintf(buf, " Total %d entries\n", TOTAL_MENU_Points);
	strcat(s, buf);
	strcat(s, " -------------------------------------------------------\n");
	return EXIT_SUCCESS;
};



//--------------------------------------------------------------------------------------------------
//   Print a Menu selected by LevelNumber and MenuNumber
//--------------------------------------------------------------------------------------------------
int Menu_print_by_Menu ( int par_LevelNo, int par_MenuNo, char s[MAX_RS_OUTPUT_STRING_LENGTH])
{
	int  i;
	char PointNo_Buffer[10];

	strcat(s, "\n ---");
	strcat(s, Menu_getMenuTitle( par_LevelNo, par_MenuNo));
	strcat(s, "---------------------------------------\n");

	for ( i=0; i<TOTAL_MENU_Points; i++ )
	{
		if (MenuDefs[i].LevelNo == par_LevelNo &&
			MenuDefs[i].MenuNo  == par_MenuNo)
		{
			sprintf(PointNo_Buffer, " [%d] ", MenuDefs[i].PointNo);
			strcat(s, PointNo_Buffer);
			strcat(s, MenuDefs[i].PointText);
			strcat(s, "\n");
		};
	}
	strcat(s, " --------------------------- [a] -> one Menu up --------\n");
	return EXIT_SUCCESS;
};



//--------------------------------------------------------------------------------------------------
//  return a Menutitle as a string
//--------------------------------------------------------------------------------------------------
char *Menu_getMenuTitle(int par_LevelNo, int par_MenuNo)
{

	int  i;
	static char s[MAX_MENU_TITLE_STRING_LENGTH];

 	s[0] = '\0';

	for ( i=0; i<=TOTAL_MENU_Points; i++ )
	{
		if (MenuDefs[i].LevelNo == par_LevelNo &&
			MenuDefs[i].MenuNo  == par_MenuNo )
		{
			strcat(s, MenuDefs[i].MenuText);
			break;
		};
	}
	return s;

}

      

//--------------------------------------------------------------------------------------------------
//  State machine for the Menunavigation
//--------------------------------------------------------------------------------------------------
int Menu_handler( void )
{
	static enum enum_states  menu_state = ST_INIT;

	switch( menu_state )
	{
	case 	ST_INIT:
			return Menu_do_init    ( &menu_state, RS_OUT_Buffer );   // Initialisation Sequence
		break;

	case 	ST_Menu_0:
			return Menu_do_Menu_0  ( &menu_state, RS_OUT_Buffer );   // Initialisation Sequence
		break;

	case 	ST_Menu_1:
			return Menu_do_Menu_1  ( &menu_state, RS_OUT_Buffer );   // Initialisation Sequence
		break;

	case 	ST_Menu_1_1:
			return Menu_do_Menu_1_1( &menu_state, RS_OUT_Buffer );  // Initialisation Sequence
		break;

	case 	ST_Menu_1_2:
			return Menu_do_Menu_1_2( &menu_state, RS_OUT_Buffer );  // Initialisation Sequence
		break;

	case 	ST_Menu_2:
			return Menu_do_Menu_2  ( &menu_state, RS_OUT_Buffer );  // Initialisation Sequence
		break;

	case 	ST_Menu_3:
			return Menu_do_Menu_3  ( &menu_state, RS_OUT_Buffer );  // Initialisation Sequence
		break;

	case 	ST_Menu_4:
			return Menu_do_Menu_4  ( &menu_state, RS_OUT_Buffer );  // Initialisation Sequence
		break;

	case 	ST_Menu_5:
			return Menu_do_Menu_5( &menu_state, RS_OUT_Buffer );   // Initialisation Sequence
		break;

	default:
			menu_state = ST_INIT;
			return EXIT_FAILURE;
		break;
	}
}   // END menu_handler



//--------------------------------------------------------------------------------------------------
//  initialision of the Output and StateMachine
//--------------------------------------------------------------------------------------------------
static int Menu_do_init( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	strcpy( MenuString, "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n" );

	strcat( MenuString, " ------------------------------------------------------- \n");
	strcat( MenuString, " |  Techniker Schule Uster                     TSU E99 | \n");
	strcat( MenuString, " |  H. Baumgartner / B. Huebscher          Version 0.8 | \n");
	strcat( MenuString, " ------------------------------------------------------- \n");
	strcat( MenuString, " |                                                     | \n");
	strcat( MenuString, " |  *   *   ***            *   *  *****  *   *  *   *  | \n");
	strcat( MenuString, " |  *   *  *   *           ** **  *      **  *  *   *  | \n");
	strcat( MenuString, " |  *****   ***    *****   * * *  ***    * * *  *   *  | \n");
	strcat( MenuString, " |  *   *  *   *           *   *  *      *  **  *   *  | \n");
	strcat( MenuString, " |  *   *   ***            *   *  *****  *   *   ***   | \n");
	strcat( MenuString, " |                                                     | \n");
	strcat( MenuString, " |                      Options:   a -> one Menu back  | \n");
	strcat( MenuString, " ------------------------------------------------------- \n");
	strcat( MenuString, "\n");

	Menu_print_by_Menu ( 1, 0, MenuString);     // Menu 0 on Level 1

	*state = ST_Menu_0;           // next State

	return EXIT_SUCCESS;
}   // END Menu_do_init




//--------------------------------------------------------------------------------------------------
//   H8-Menu - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_0( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  Menu_print_by_Menu ( 2, 1, MenuString);  *state = ST_Menu_1;	break;
	case 	'2':  Menu_print_by_Menu ( 2, 2, MenuString);  *state = ST_Menu_2;	break;
	case 	'3':  Menu_print_by_Menu ( 2, 3, MenuString);  *state = ST_Menu_3;	break;
	case 	'4':  Menu_print_by_Menu ( 2, 4, MenuString);  *state = ST_Menu_4;	break;
	case 	'5':  Menu_print_by_Menu ( 2, 5, MenuString);  *state = ST_Menu_5;	break;

	default:      Menu_do_init( state, MenuString );  *state = ST_Menu_0;	    break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_0




//--------------------------------------------------------------------------------------------------
//  LED's - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_1( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  Menu_print_by_Menu ( 3, 11, MenuString);  *state = ST_Menu_1_1;	break;   // LED's green
	case 	'2':  Menu_print_by_Menu ( 3, 12, MenuString);  *state = ST_Menu_1_2;	break;   // LED's red

	case 	'a':  Menu_print_by_Menu ( 1, 0, MenuString);  *state = ST_Menu_0;	    break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_1




//--------------------------------------------------------------------------------------------------
//  LED's green - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_1_1( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  LED_Gruen_ausg ('1');   break;       // LED green 0 
	case 	'2':  LED_Gruen_ausg ('2');   break;       // LED green 1
	case 	'3':  LED_Gruen_ausg ('3');   break;       // LED green 2
	case 	'4':  LED_Gruen_ausg ('4');   break;       // LED green 3
	case 	'5':  LED_Gruen_ausg ('5');   break;       // LED green 4
	case 	'6':  LED_Gruen_ausg ('6');   break;       // LED green 5
	case 	'7':  LED_Gruen_ausg ('7');   break;       // LED green 6
	case 	'8':  LED_Gruen_ausg ('8');   break;       // LED green 7
	case 	'0':  LED_Gruen_ausg ('0');   break;       // all LED's green off 

	case 	'a':  Menu_print_by_Menu ( 2, 1, MenuString);  *state = ST_Menu_1;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_1_1



//--------------------------------------------------------------------------------------------------
//  LED's red - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_1_2( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  LED_Rot_ausg ('1');   break;       // LED red 0 
	case 	'2':  LED_Rot_ausg ('2');   break;       // LED red 1
	case 	'3':  LED_Rot_ausg ('3');   break;       // LED red 2
	case 	'4':  LED_Rot_ausg ('4');   break;       // LED red 3
	case 	'5':  LED_Rot_ausg ('5');   break;       // LED red 4
	case 	'6':  LED_Rot_ausg ('6');   break;       // LED red 5
	case 	'7':  LED_Rot_ausg ('7');   break;       // LED red 6
	case 	'8':  LED_Rot_ausg ('8');   break;       // LED red 7
	case 	'0':  LED_Rot_ausg ('0');   break;       // all LED's red off 

	case 	'a':  Menu_print_by_Menu ( 2, 1, MenuString);  *state = ST_Menu_1;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_1_2



//--------------------------------------------------------------------------------------------------
//  Switches - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_2( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  Switch_anz('0');    break;       // Display SW - Decimal
	case 	'2':  Switch_anz('2');    break;       // Display SW - Hexadecimal
	case 	'3':  Switch_anz('1');    break;       // Display SW - Binary
	case 	'4':  break;                           // Display HexSW - Decimal
	case 	'5':  break;                           // Display HexSW - Hexadecimal
	case 	'6':  break;                           // Display HexSW - Binary

	case 	'a':  Menu_print_by_Menu ( 1, 0, MenuString);  *state = ST_Menu_0;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_2



//--------------------------------------------------------------------------------------------------
//   LCD-Display - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_3( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  PBDR |= 0x20; break;   // Backlight einschalten
	case 	'2':  PBDR &= !0x20; break;  // Backlight ausschalten   
	case 	'3':  break;
	case 	'4':  break;
	case 	'5':  break;
	case 	'6':  break;

	case 	'a':  Menu_print_by_Menu ( 1, 0, MenuString);  *state = ST_Menu_0;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_3



//--------------------------------------------------------------------------------------------------
//  Keyboard - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_4( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1':  Keyboard_disp();   break;
	case 	'2':  break;
	case 	'3':  break;
	case 	'4':  break;

	case 	'a':  Menu_print_by_Menu ( 1, 0, MenuString);  *state = ST_Menu_0;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_4



//--------------------------------------------------------------------------------------------------
// Menu Infos - Menu
//--------------------------------------------------------------------------------------------------
static int Menu_do_Menu_5( enum enum_states *state, char MenuString[MAX_RS_OUTPUT_STRING_LENGTH] )
{
	// be aware of MAX_RS_OUTPUT_STRING_LENGTH !!!

	switch( RS_IN_Buffer[0] )
	{
	case 	'1': Menu_print_all( RS_OUT_Buffer);     
					break;                                      // Print hole Menu
	case 	'2':  strcat( RS_OUT_Buffer, "\n -> H8-menu created by: Benno Huebscher (TauGnix)");
	              strcat( RS_OUT_Buffer, "\n                        Hanno Baumgartner\n");
	              strcat( RS_OUT_Buffer, "\n    December 2001 TSU         Version 0.8\n");
					break;                                      // Print about
	case 	'3':  CBTSU_DispString(2, LCDTimeBuffer);
				  strcat( RS_OUT_Buffer, LCDTimeBuffer);
					break;
	case 	'4':  break;

	case 	'a':  Menu_print_by_Menu ( 1, 0, MenuString);  *state = ST_Menu_0;	break;

	default:

		break;
	}
	return EXIT_SUCCESS;
}   // END Menu_do_Menu_5




//--------------------------------------------------------------------------------------------------
// Funktion zur Prfung der roten LED Anzeige
//--------------------------------------------------------------------------------------------------
void LED_Rot_ausg (char LED_contr)        // Funktion zur Prfung der roten LED Anzeige
{
BYTE  z;
BYTE  LED;

if ((LED_contr > 47) && (LED_contr < 57)) // Prfen, LED welche eingeschaltet werden soll
  {                                       // gltig ist
  LED_contr = LED_contr-48;               // umrechnung des ASCI Zeichen in Integer
  if (LED_contr==0)
    {
    LED = 0;                              // Ist das Steuerzeichen = 0, dann werden alle LED
    }                                     // ausgeschaltet
  else 
    {
    LED=1;                                // ansonsten wird diejenige LED eingeschaltet, welche
    for (z=1; z<LED_contr; z++)           // dem Steuerzeichen Entspricht:
      LED = LED*2;                        // BSP: Steuerzeichen = 1 entspricht der 1. LED
    }
  LEDrot = (BYTE)LED;                     // Ausgeben des Registerwertes an LED
  sprintf(buffer,"LED Rot : %u       ",LED_contr); // Ausgabe auf Display, welche rote LED
  CBTSU_DispString(3,buffer);             // nun aufleuchten soll
  }
}


//--------------------------------------------------------------------------------------------------
// Funktion zur Prfung der grnen LED Anzeige
//--------------------------------------------------------------------------------------------------
void LED_Gruen_ausg (char LED_contr)  
{
BYTE  z;
BYTE  LED;

if ((LED_contr > 47) && (LED_contr < 57)) // Prfen, LED welche eingeschaltet werden soll
  {                                       // gltig ist
  LED_contr = LED_contr-48;               // umrechnung des ASCI Zeichen in Integer
  if (LED_contr==0)
    {
    LED = 0;                              // Ist das Steuerzeichen = 0, dann werden alle LED
    }                                     // ausgeschaltet
  else 
    {
    LED=1;                                // ansonsten wird diejenige LED eingeschaltet, welche
    for (z=1; z<LED_contr; z++)           // dem Steuerzeichen Entspricht:
      LED = LED*2;                        // BSP: Steuerzeichen = 1 entspricht der 1. LED
    }
  LEDgrun = (BYTE)LED;                    // Ausgeben des Registerwertes an LED
  sprintf(buffer,"LED Gruen : %u       ",LED_contr);  // Ausgabe auf Display, welche grne LED
  CBTSU_DispString(3,buffer);             // nun aufleuchten soll
  }
}


//--------------------------------------------------------------------------------------------------
// Funktion zum darstellen der Schalterpositon auf Display (hex, dez, bin)
//--------------------------------------------------------------------------------------------------
void Switch_anz (char Switch_contr)
{
	BYTE  Sw_pos;
	BYTE  i; 
	BYTE  dual;
	
	Sw_pos = Switch;          // Schalterstellung in Variabel Sichern
	dual = 128;               // Anfangswert fr Ausmaskierung der Bits setzen
	switch (Switch_contr)     // Anzeige auf Display anhand Kontrollbefehl
	{
		case '0' :
			sprintf(buffer,"Switch(dez): %03u  ",Sw_pos); // Ausgabe der Schalter-
			break;

		case '1' :  
			sprintf(buffer,"Sw (bin): xxxxxxxx");         // Ausgabe der Schalter-
			for (i=10; i<18; i++)         // Stellung auf Display in Bin
			{                             // Ausmaskierung der einzelnen Schalter
				if(Sw_pos & dual)         // Ist der jeweilige Schalter gesetzt, so
				{                         // wird im Ausgabestring eine 1 gespeichert
					buffer[i] = '1';
				}
				else
				{
					buffer[i] = '0';      // ansonsten wird im Ausgabstring
				}                         // eine 0 gespeichert 
				dual = dual / 2;          // Ausmaskierungswert aufs nchste niederwertigere 
			}                             // Bit setzen

			break;
		case '2' :
			sprintf(buffer,"Switch(hex): 0x%02X ",Sw_pos); // Ausgabe der Schalter-

			break;
	}
	CBTSU_DispString(3,buffer);           // Stellung auf LCD-Display
	strcpy( RS_OUT_Buffer, "   -> " );    // ausgabe auf serielle Schnittstelle
	strcat( RS_OUT_Buffer, buffer );      // ausgabe auf serielle Schnittstelle
	
}   // END Switch_anz 


//--------------------------------------------------------------------------------------------------
// Funktion zum darstellen der gedrckten Taste auf Tastatur
//--------------------------------------------------------------------------------------------------
void Keyboard_disp (void)
{
	sprintf(buffer,"Key:                ");    // Grundausgabe fr KEY in buffer schreiben
	buffer[5] = KeyboardVal;                   // Tastenzeichen (Wert) in buffer kopieren
	CBTSU_DispString(3,buffer);                // Stellung auf LCD-Display
	strcpy( RS_OUT_Buffer, "   -> " );         // ausgabe auf serielle Schnittstelle
	strcat( RS_OUT_Buffer, buffer );           // ausgabe auf serielle Schnittstelle
	
}   // END Keyboard_disp 

//--------------------------------------------------------------------------------------------------
// end of code
//--------------------------------------------------------------------------------------------------
